
from genshi.builder import tag
from trac.core import *
from trac.web.href import Href
try:
    from trac.search import ISearchSource, shorten_result
except ImportError:
    from trac.Search import ISearchSource, shorten_result

try:
    from trac.search.web_ui import SearchModule
except ImportError:
    from trac.Search import SearchModule

from trac.perm import IPermissionRequestor
from trac.env import open_environment
from trac.util.text import shorten_line
import re
import posixpath
import os
import logging
import copy


from fnmatch import fnmatch

class SearchAllPlugin(Component):
    """ Search the source repository. """
    implements(ISearchSource)

    # ISearchSource methods
    def get_search_filters(self, req):
        yield ('searchall', 'All projects', 0)
        yield ('searchrepo', 'svn repo', 0)

    def get_search_results(self, req, query, filters):
        #return if search all is not active
        if 'searchall' in filters:
            for val in self.get_searchall_results(req, query, filters):
                yield val

        if 'searchrepo' in filters:
            for val in self.get_searchrepo_results(req, query, filters):
                yield val
    
    def get_searchrepo_results (self, req, query, filters):
        self.env.log.debug("Searching the repo as %s for: %s" % (req.authname, query))
        expr = re.compile(' '.join(query), re.IGNORECASE | re.MULTILINE | re.UNICODE)
        repos = self.env.get_repository(req.authname)
        revs = range(1, repos.get_youngest_rev())
        revs.reverse() #go newest to first
        def handle_revision(rev, repos):
            changeset = repos.get_changeset(rev)
            changes = changeset.get_changes()
            for change in changes:
                self.env.log.debug("About to interact with change from rev: %s :\n %s" % (rev, change))
                (newpath, type, changetype, oldpath, lastrev) = change
                if type == 'file' and changetype != 'delete':
                    node = repos.get_node(newpath, rev)
                    if node.get_content_type()!='application/octet-stream':
                        contentstream = node.get_content()
                        if contentstream:
                            content = contentstream.read()
                            if content and expr.search(content):
                                return (req.href.changeset(rev),
                                        '[%s]: %s' % (rev, shorten_line(changeset.message)),
                                        changeset.date, changeset.author,
                                        shorten_result(changeset.message, query))
            
        for rev in revs:
            rtn = handle_revision(rev, repos)
            if rtn:
                yield rtn
            
                        


    def get_searchall_results (self, req, query, filters):
        # get search path and base_url
        search_path, this_project = os.path.split(self.env.path)
        base_url, _ = posixpath.split(req.abs_href())
            
        # remove 'meta' from filters
        subfilters = [];
        for filter in filters:
           if not filter == 'searchall':
               subfilters.append( filter )      
               
        for project in os.listdir(search_path):
            # skip our own project
            if project == this_project:
                continue
            
            # make up URL for project
            project_url = '/'.join( (base_url, project) )  
            
            project_path = os.path.join(search_path,project)
            if not os.path.isdir( project_path ):
                continue
            try:
                env = open_environment(project_path)
            except:
                continue
                        
           
            self.env.log.debug("Searching project %s" % project )
            self.env.log.debug("Searching for %s" % query[0] )           
            self.env.log.debug("Searching with filters %s" % subfilters )

            # don't do anything if we have no filters
            if not subfilters:
                continue
                
            results = []
            env_search = SearchModule(env)
            
            #Update request data
            orig_href = req.href
            req.href = Href(project_url)
            
            for source in env_search.search_sources:
                results += list(source.get_search_results(req, query, subfilters))
            
            req.href = orig_href
            
            for result in results:
                yield (result[0], tag(env.project_name, ': ', result[1])) + result[2:]

